package com.stormtales.notevault.ui.login;

import android.util.Log;
import android.util.Patterns;
import android.widget.Toast;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;
import com.stormtales.notevault.R;
import com.stormtales.notevault.network.APICallback;
import com.stormtales.notevault.network.auth.AuthService;
import com.stormtales.notevault.network.auth.LoginResponse;

public class LoginViewModel extends ViewModel {

    private final MutableLiveData<String> username;
    private final MutableLiveData<Boolean> isLoggedIn;
    private MutableLiveData<LoginFormState> loginFormState = new MutableLiveData<>();
    private AuthService authService;

    public LoginViewModel() {
        username = new MutableLiveData<>("Not Logged In");
        isLoggedIn = new MutableLiveData<>(false);
    }

    public void performLogin(String email, String password, SuccessFullLoginCallback successFullLoginCallback) {
        if(authService != null) {
            authService.performLogin(email, password, new LoginCallBackImpl(successFullLoginCallback));
        } else {
            throw new RuntimeException("AuthService is not provided to LoginViewModel");
        }
    }

    public void performRegistration(String email, String password, String username, SuccessFullLoginCallback successFullLoginCallback) {
        if(authService != null) {
            authService.performRegistration(email, username, password, new APICallback() {
                @Override
                public void onSuccess() {
                    successFullLoginCallback.onSuccess();
                }

                @Override
                public void onError(String error) {
                    Log.d("LoginService", error);
                }
            });
        }
    }

    public void setAuthService(AuthService authService) {
        this.authService = authService;
    }

    public MutableLiveData<Boolean> getIsLoggedIn() {
        return isLoggedIn;
    }

    public MutableLiveData<String> getUsername() {
        return username;
    }

    public void updateLoginData(String username, String password) {
        if(!isUserNameValid(username)) {
            loginFormState.setValue(new LoginFormState(R.string.invalid_username, null));
        } else if(!isPasswordValid(password)) {
            loginFormState.setValue(new LoginFormState(null, R.string.invalid_password));
        } else {
            loginFormState.setValue(new LoginFormState(true));
        }
    }

    public class LoginCallBackImpl implements AuthService.LoginCallback {

        private final SuccessFullLoginCallback successFullLoginCallback;

        public LoginCallBackImpl(SuccessFullLoginCallback successFullLoginCallback) {
            this.successFullLoginCallback = successFullLoginCallback;
        }

        @Override
        public void onSuccess(LoginResponse loginResponse) {
            username.setValue(loginResponse.getUsername());
            isLoggedIn.setValue(true);
            successFullLoginCallback.onSuccess();
        }

        @Override
        public void onError(String error) {
            Log.d("LoginService", error);
        }
    }

    public interface SuccessFullLoginCallback {
        void onSuccess();
    }

    public MutableLiveData<LoginFormState> getLoginFormState() {
        return loginFormState;
    }

    // A placeholder username validation check
    private boolean isUserNameValid(String username) {
        if (username == null) {
            return false;
        }
        if (username.contains("@")) {
            return Patterns.EMAIL_ADDRESS.matcher(username).matches();
        } else {
            return !username.trim().isEmpty();
        }
    }

    // A placeholder password validation check
    private boolean isPasswordValid(String password) {
        return password != null && password.trim().length() > 5;
    }
}
